// src/main.rs
#![no_std]
#![no_main]
#![feature(naked_functions)]

use core::panic::PanicInfo;

mod entry;
mod heap;
mod trap;
mod uart;

core::arch::global_asm!(include_str!("entry.S"));

#[no_mangle]
extern "C" fn start() -> ! {
    // UNSAFE: Called exactly once, right here.
    unsafe { heap::init() };

    // Now we're free to use dynamic allocation!
    // {
    //     extern crate alloc;
    //     use alloc::boxed::Box;
    //     let _my_heap_pointer = Box::new(10);
    //     // _my_heap_pointer lives on the heap!
    // }

    // println!("This should not print because the console is not initialised.");
    uart::init_console(0x1000_0000);
    println!("Hello, world!");

    // print current hartid
    println!("hartid: {}", riscv::register::mhartid::read());

    // poll console for input and print characters back
    loop {
        let c = uart::CONSOLE.lock().as_mut().and_then(uart::Device::get);
        if let Some(c) = c { print!("{}", c as char); }
    }
}

#[no_mangle]
extern "C" fn eh_personality() {}

#[panic_handler]
fn panic(info: &PanicInfo) -> ! {
    // print panic info and abort
    println!("{}", info);
    abort();
}

#[no_mangle]
extern "C" fn abort() -> ! {
    // loop wait for interrupt
    loop { riscv::asm::wfi(); }
}

// TODO unit testing